/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of rrd_agent.
	
    rrd_agent is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    rrd_agent is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#include "config.h"
#include <sstream>
#include <fstream>
#include <vector>
#include <string.h>

#include <boost/scoped_array.hpp>
#include <boost/shared_array.hpp>
#include <rrd.h>
#include <log4cplus/logger.h>

std::string get_filename(const std::string & data_folder, const std::string & logger_code, const std::string & monitor_code, int channel_number);
bool file_exists(const std::string & filename) ;

bool create_rrd(const std::string & filename, const std::string & rra_defs, const std::string & time, const std::string & dsname, int steps) {	
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	std::stringstream stream;
	stream << "DS:" << dsname << ":GAUGE:" << steps << ":" << "U" << ":" << "U";
	boost::shared_array<char> ds(new char[stream.str().length()+1]);
	strcpy(ds.get(), stream.str().c_str());
	time_t tm = atol(time.c_str());
	std::stringstream ts;
	ts << tm - 6;
	std::string tm_s = ts.str();
	std::ifstream rra_conf(rra_defs.c_str());
	std::vector<std::string> rra_list;	
	std::string s;
	while(rra_conf >> s) {
		rra_list.push_back(s);
	}
	
	boost::shared_array< const char* > opts(new const char * [1+rra_list.size()]);
	opts[0] = ds.get();
	for(unsigned int i=0; i < rra_list.size(); ++i) {
		opts[i+1] = rra_list[i].c_str(); 
	}
	int opts_size = rra_list.size();
	int ret = rrd_create_r(filename.c_str(), 6, tm-6, opts_size, opts.get());
	
	std::ostringstream log_message;
	if(ret ==  -1) {
		log_message << "rrdtool create error (" << rrd_get_error() << ")";
		LOG4CPLUS_INFO(logger, log_message.str());
		return false;
	}
	return true;
}
bool rrd_process(const std::string & rrd_path, const std::string & rra_defs, const std::string & time, const std::string & logger_code, const std::string & monitor_code, int channel_number, const std::string & channel_type, const std::string & channel_value)
{
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	std::string filename =  get_filename(rrd_path, logger_code, monitor_code, channel_number);
	if(!file_exists(filename)) {
		create_rrd(filename, rra_defs, time, channel_type, 9);
	}
	std::ostringstream stream;
	stream << time << ":" << channel_value;
	std::string value_opt = stream.str();
	const char * opts[] = { value_opt.c_str() };

	int ret = rrd_update_r(filename.c_str(), NULL, 1, opts);
	std::ostringstream log_message;
	if(ret ==  -1) {
		log_message << "rrdtool update error (" << rrd_get_error() << "[" << filename << ":" << value_opt << "])";
		LOG4CPLUS_ERROR(logger, log_message.str());
		return false;
	}
	else {
		log_message.str("");
		log_message << "update (" << value_opt << ")";
		LOG4CPLUS_DEBUG(logger, log_message.str());
	}
	return true;
}

